var get_list_of_primary_conditions = function () {
    var primary_conditions = appData.primary_conditions;
    var list_of_primary_conditions = [];

    for (var primary_condition in primary_conditions) {
        list_of_primary_conditions.push(primary_conditions[primary_condition].name);
    }

    return list_of_primary_conditions;
};

var get_primary_condition_obj_by_name = function (primary_condition_name) {
    var primary_conditions = appData.primary_conditions;
    var obj = null;

    for (var i = 0; i < primary_conditions.length; i++) {
        if (primary_conditions[i].name === primary_condition_name) {
            obj = primary_conditions[i];
            break;
        }
    }

    return obj;
};

var get_available_sub_conditions_for_primary_condition = function (primary_condition_name) {
    if (primary_condition_name) {
        var primary_conditions_obj = get_primary_condition_obj_by_name(primary_condition_name);
        return primary_conditions_obj.available_sub_conditions;
    }

    return null;
};

var get_available_actions_for_primary_condition = function (primary_condition_name) {
    if (primary_condition_name) {
        var primary_conditions_obj = get_primary_condition_obj_by_name(primary_condition_name);
        return primary_conditions_obj.available_actions;
    }

    return null;
};

var set_all_primary_conditions_container_hidden = function () {
    templateObj.$.ti_widget_container_primary_condition_empty.visible = true;

    templateObj.$.ti_widget_container_gpio_input.visible = false;
    templateObj.$.ti_widget_container_system_event.visible = false;
    templateObj.$.ti_widget_container_counter_threshold.visible = false;
    templateObj.$.ti_widget_container_internal_error.visible = false;
    templateObj.$.ti_widget_container_timer_expiration.visible = false;
    templateObj.$.ti_widget_container_pre_init_event.visible = false;
};

var onPrimaryConditionSelected = function () {
    var device_type = templateObj.$.ti_widget_droplist_device_type.selectedText;

    var selectedValue = templateObj.$.ti_widget_droplist_primary_conditions.selectedText;
    console.log(selectedValue);

    set_all_primary_conditions_container_hidden();

    if (selectedValue) {
        templateObj.$.ti_widget_container_primary_condition_empty.visible = false;

        switch (selectedValue) {
            case "GPIO Input": // GPIO Input
                templateObj.$.ti_widget_droplist_gpio_in_pad.labels = get_input_pad_numbers(device_type);

                templateObj.$.ti_widget_container_gpio_input.style.left = primary_condition_pos_left;
                templateObj.$.ti_widget_container_gpio_input.style.top = primary_condition_pos_top;

                templateObj.$.ti_widget_container_gpio_input.visible = true;
                break;
            case "System Event": // System Event
                templateObj.$.ti_widget_droplist_sys_event_type.labels = get_system_events_list().join("|");

                templateObj.$.ti_widget_container_system_event.style.left = primary_condition_pos_left;
                templateObj.$.ti_widget_container_system_event.style.top = primary_condition_pos_top;

                templateObj.$.ti_widget_container_system_event.visible = true;
                break;
            case "Counter threshold": // Counter threshold
                templateObj.$.ti_widget_droplist_ct_counter_number.labels = getAllUsedCounterNumbers();

                templateObj.$.ti_widget_container_counter_threshold.style.left = primary_condition_pos_left;
                templateObj.$.ti_widget_container_counter_threshold.style.top = primary_condition_pos_top;

                templateObj.$.ti_widget_container_counter_threshold.visible = true;
                break;
            case "Internal Error": // Internal Error
                templateObj.$.ti_widget_container_internal_error.style.left = primary_condition_pos_left;
                templateObj.$.ti_widget_container_internal_error.style.top = primary_condition_pos_top;

                templateObj.$.ti_widget_container_internal_error.visible = true;
                break;
            case "Timer Expiration": // Timer Expiration
                templateObj.$.ti_widget_droplist_timer_expiration_number.labels = getAllUsedTimerNumbers();

                templateObj.$.ti_widget_container_timer_expiration.style.left = primary_condition_pos_left;
                templateObj.$.ti_widget_container_timer_expiration.style.top = primary_condition_pos_top;

                templateObj.$.ti_widget_container_timer_expiration.visible = true;
                break;
            case "Pre-init event": // Pre-init event
                templateObj.$.ti_widget_container_pre_init_event.style.left = primary_condition_pos_left;
                templateObj.$.ti_widget_container_pre_init_event.style.top = primary_condition_pos_top;

                templateObj.$.ti_widget_container_pre_init_event.visible = true;
                break;
        }

        // Update avaliale sub condition for selected primary condition
        var available_sub_conditions = get_available_sub_conditions_for_primary_condition(selectedValue);
        if (available_sub_conditions) {
            templateObj.$.ti_widget_droplist_sub_conditions.labels = available_sub_conditions.join('|');
        } else {
            templateObj.$.ti_widget_droplist_sub_conditions.labels = get_list_of_sub_conditions().join('|');
        }

        // templateObj.$.ti_widget_droplist_sub_conditions.selectedIndex = (available_sub_conditions && available_sub_conditions.length === 1) ? 0 : -1;
        templateObj.$.ti_widget_droplist_sub_conditions.selectedIndex = -1;
        onSubConditionSelected();


        // Update avaliale actions for selected primary condition
        var available_actions = get_available_actions_for_primary_condition(selectedValue);
        if (available_actions) {
            templateObj.$.ti_widget_droplist_actions.labels = available_actions.join('|');
        } else {
            templateObj.$.ti_widget_droplist_actions.labels = get_list_of_actions().join('|');
        }

        // templateObj.$.ti_widget_droplist_actions.selectedIndex = (available_actions && available_actions.length === 1) ? 0 : -1;
        templateObj.$.ti_widget_droplist_actions.selectedIndex = -1;
        onActionSelected();
    }
};

var getDataForPrimaryCondition = function (rule) {
    var selectedIndex = templateObj.$.ti_widget_droplist_primary_conditions.selectedIndex;
    var selectedText = templateObj.$.ti_widget_droplist_primary_conditions.selectedText;

    if (selectedIndex === -1) {
        alert("Please select Primary Condition before continue forward");
        return undefined;
    }

    var primary_conditions_obj = get_primary_condition_obj_by_name(selectedText);

    rule.primary_conditions_type = {
        "name": "primary_conditions_type",
        "raw_val": selectedIndex,
        "val": primary_conditions_obj.id,
        "size": 4
    };

    rule.primary_conditions_length = {
        "name": "primary_conditions_length",
        "raw_val": "",
        "val": primary_conditions_obj.length,
        "size": 4
    };

    rule.primary_conditions_params = [];

    switch (selectedText) {
        case "GPIO Input":
            var pad_number = {
                "name": "pad_number",
                "raw_val": templateObj.$.ti_widget_droplist_gpio_in_pad.slectedIndex,
                "val": templateObj.$.ti_widget_droplist_gpio_in_pad.selectedText,
                "size": 1
            };
            rule.primary_conditions_params.push(pad_number);

            var edge_level = {
                "name": "edge_level",
                "raw_val": templateObj.$.ti_widget_droplist_gpio_in_level.slectedIndex,
                "val": templateObj.$.ti_widget_droplist_gpio_in_level.selectedIndex.toString(),
                "size": 1
            };
            rule.primary_conditions_params.push(edge_level);

            var pull = {
                "name": "pull",
                "raw_val": templateObj.$.ti_widget_droplist_gpio_in_pull.slectedIndex,
                "val": templateObj.$.ti_widget_droplist_gpio_in_pull.selectedIndex.toString(),
                "size": 1
            };
            rule.primary_conditions_params.push(pull);

            var retention = {
                "name": "retention",
                "raw_val": templateObj.$.ti_widget_droplist_gpio_in_retention.slectedIndex,
                "val": templateObj.$.ti_widget_droplist_gpio_in_retention.selectedIndex.toString(),
                "size": 1
            };
            rule.primary_conditions_params.push(retention);

            break;

        case "System Event":
            var system_event = {
                "name": "system_event_id",
                "raw_val": templateObj.$.ti_widget_droplist_sys_event_type.selectedText,
                "val": get_system_event_value_by_name(templateObj.$.ti_widget_droplist_sys_event_type.selectedText),
                "size": 2
            };
            rule.primary_conditions_params.push(system_event);

            var system_event_padding = {
                "name": "zero_pad",
                "raw_val": "",
                "val": "",
                "size": 2
            };
            rule.primary_conditions_params.push(system_event_padding);

            break;

        case "Counter threshold":
            var threshold_value = {
                "name": "threshold_value",
                "raw_val": templateObj.$.ti_widget_numberbox_counter_threshold.value,
                "val": "0x" + templateObj.$.ti_widget_numberbox_counter_threshold.value.toString(16),
                "size": 4
            };
            rule.primary_conditions_params.push(threshold_value);

            var selected_counter_number = templateObj.$.ti_widget_droplist_ct_counter_number.selectedText;
            if ( selected_counter_number === undefined) {
                alert("Please select counter number before saving rule")
                return undefined;
            }

            var counter_number = {
                "name": "counter_number",
                "raw_val": selected_counter_number,
                "val": "0x" + selected_counter_number.toString(16),
                "size": 2
            };
            rule.primary_conditions_params.push(counter_number);

            var counter_threshold_padding = {
                "name": "zero_pad",
                "raw_val": "",
                "val": "",
                "size": 2
            };
            rule.primary_conditions_params.push(counter_threshold_padding);

            break;

        case "Internal Error":
            var fatal = {
                "name": "fatal",
                "raw_val": templateObj.$.ti_widget_droplist_internal_error_fatal.selectedIndex,
                "val": templateObj.$.ti_widget_droplist_internal_error_fatal.selectedIndex.toString(),
                "size": 1
            };
            rule.primary_conditions_params.push(fatal);

            var internal_error_padding = {
                "name": "zero_pad",
                "raw_val": "",
                "val": "",
                "size": 3
            };
            rule.primary_conditions_params.push(internal_error_padding);

            break;

        case "Timer Expiration":
            var timer_expiration_options = {
                "name": "zero pad",
                "raw_val": "",
                "val": "",
                "size": 2
            };
            rule.primary_conditions_params.push(timer_expiration_options);

            var selected_timer_number = templateObj.$.ti_widget_droplist_timer_expiration_number.selectedText;
            if ( selected_timer_number === undefined) {
                alert("Please select timer number before saving rule")
                return undefined;
            }

            var timer_number = {
                "name": "timer_number",
                "raw_val": selected_timer_number,
                "val": "0x" + selected_timer_number.toString(16),
                "size": 1
            };
            rule.primary_conditions_params.push(timer_number);

            var timer_expiration_padding = {
                "name": "zero_pad",
                "raw_val": "",
                "val": "",
                "size": 1
            };
            rule.primary_conditions_params.push(timer_expiration_padding);

            break;

        case "Pre-init event":
            var preinit_event_padding = {
                "name": "zero_pad",
                "raw_val": "",
                "val": "",
                "size": 4
            };
            rule.primary_conditions_params.push(preinit_event_padding);

            break;
    }

    return rule;
};

var loadDataForPrimaryCondition = function (rule) {
    templateObj.$.ti_widget_droplist_primary_conditions.selectedIndex = rule.primary_conditions_type.raw_val;
    onPrimaryConditionSelected();

    var selectedText = templateObj.$.ti_widget_droplist_primary_conditions.selectedText;
    switch (selectedText) {
        case "GPIO Input":
            templateObj.$.ti_widget_droplist_gpio_in_pad.slectedIndex = get_value_by_name(rule.primary_conditions_params, "pad_number");

            templateObj.$.ti_widget_droplist_gpio_in_level.slectedIndex = get_value_by_name(rule.primary_conditions_params, "edge_level");

            templateObj.$.ti_widget_droplist_gpio_in_pull.slectedIndex = get_value_by_name(rule.primary_conditions_params, "pull");

            templateObj.$.ti_widget_droplist_gpio_in_retention.slectedIndex = get_value_by_name(rule.primary_conditions_params, "retention");

            break;

        case "System Event":
            templateObj.$.ti_widget_droplist_sys_event_type.selectedText = get_value_by_name(rule.primary_conditions_params, "system_event_id");

            break;

        case "Counter threshold":
            templateObj.$.ti_widget_numberbox_counter_threshold.value = get_value_by_name(rule.primary_conditions_params, "threshold_value");

            templateObj.$.ti_widget_droplist_ct_counter_number.selectedText = get_value_by_name(rule.primary_conditions_params, "counter_number");

            break;

        case "Internal Error":
            templateObj.$.ti_widget_droplist_internal_error_fatal.selectedIndex = get_value_by_name(rule.primary_conditions_params, "fatal");

            break;

        case "Timer Expiration":
            templateObj.$.ti_widget_droplist_timer_expiration_number.selectedText = get_value_by_name(rule.primary_conditions_params, "timer_number");
            
            break;

        case "Pre-init event":
            break;
    }

};
